@@ -1,5 +1,6 @@
 from math import pi
 from collections import namedtuple
+from typing import Tuple
 
 import scipy.signal
 import torch
@@ -61,6 +62,7 @@ class SphericalTensor(o3.Irreps):
     >>> SphericalTensor(3, 1, -1)
     1x0e+1x1o+1x2e+1x3o
     """
+
     # pylint: disable=abstract-method
 
     def __new__(
@@ -190,7 +192,7 @@ class SphericalTensor(o3.Irreps):
 
         return 4 * pi / (self.lmax + 1) ** 2 * (y * v).sum(-2)
 
-    def from_samples_on_s2(self, positions: torch.Tensor, values: torch.Tensor, res=100) -> torch.Tensor:
+    def from_samples_on_s2(self, positions: torch.Tensor, values: torch.Tensor, res: int = 100) -> torch.Tensor:
         r"""Convert a set of position on the sphere and values into a spherical tensor
 
         Parameters
@@ -263,7 +265,7 @@ class SphericalTensor(o3.Irreps):
 
         return s2(val)
 
-    def norms(self, signal):
+    def norms(self, signal) -> torch.Tensor:
         r"""The norms of each l component
 
         Parameters
@@ -291,7 +293,7 @@ class SphericalTensor(o3.Irreps):
             i += ir.dim
         return torch.stack(norms, dim=-1)
 
-    def signal_xyz(self, signal, r):
+    def signal_xyz(self, signal, r) -> torch.Tensor:
         r"""Evaluate the signal on given points on the sphere
 
         .. math::
@@ -322,13 +324,15 @@ class SphericalTensor(o3.Irreps):
         output = torch.einsum("bi,ai->ab", sh.reshape(-1, dim), signal.reshape(-1, dim))
         return output.reshape(signal.shape[:-1] + r.shape[:-1])
 
-    def signal_on_grid(self, signal, res=100, normalization="integral"):
+    def signal_on_grid(self, signal, res: int = 100, normalization: str = "integral"):
         r"""Evaluate the signal on a grid on the sphere"""
         Ret = namedtuple("Return", "grid, values")
         s2 = ToS2Grid(lmax=self.lmax, res=res, normalization=normalization)
         return Ret(s2.grid, s2(signal))
 
-    def plotly_surface(self, signals, centers=None, res=100, radius=True, relu=False, normalization="integral"):
+    def plotly_surface(
+        self, signals, centers=None, res: int = 100, radius: bool = True, relu: bool = False, normalization: str = "integral"
+    ):
         r"""Create traces for plotly
 
         Examples
@@ -350,16 +354,18 @@ class SphericalTensor(o3.Irreps):
         for signal, center in zip(signals, centers):
             r, f = self.plot(signal, center, res, radius, relu, normalization)
             traces += [
-                dict(
-                    x=r[:, :, 0].numpy(),
-                    y=r[:, :, 1].numpy(),
-                    z=r[:, :, 2].numpy(),
-                    surfacecolor=f.numpy(),
-                )
+                {
+                    "x": r[:, :, 0].numpy(),
+                    "y": r[:, :, 1].numpy(),
+                    "z": r[:, :, 2].numpy(),
+                    "surfacecolor": f.numpy(),
+                }
             ]
         return traces
 
-    def plot(self, signal, center=None, res=100, radius=True, relu=False, normalization="integral"):
+    def plot(
+        self, signal, center=None, res: int = 100, radius: bool = True, relu: bool = False, normalization: str = "integral"
+    ) -> Tuple[torch.Tensor, torch.Tensor]:
         r"""Create surface in order to make a plot"""
         assert signal.dim() == 1
 
@@ -384,7 +390,7 @@ class SphericalTensor(o3.Irreps):
 
         return r, f
 
-    def find_peaks(self, signal, res=100):
+    def find_peaks(self, signal, res: int = 100) -> Tuple[torch.Tensor, torch.Tensor]:
         r"""Locate peaks on the sphere
 
         Examples
