@@ -1,6 +1,7 @@
 """
 >>> test()
 """
+
 import torch
 from e3nn import o3
 from e3nn.nn import Gate
@@ -8,7 +9,7 @@ from e3nn.nn import Gate
 from .points_convolution import Convolution
 
 
-def tp_path_exists(irreps_in1, irreps_in2, ir_out):
+def tp_path_exists(irreps_in1, irreps_in2, ir_out) -> bool:
     irreps_in1 = o3.Irreps(irreps_in1).simplify()
     irreps_in2 = o3.Irreps(irreps_in2).simplify()
     ir_out = o3.Irrep(ir_out)
@@ -21,7 +22,7 @@ def tp_path_exists(irreps_in1, irreps_in2, ir_out):
 
 
 class Compose(torch.nn.Module):
-    def __init__(self, first, second):
+    def __init__(self, first, second) -> None:
         super().__init__()
         self.first = first
         self.second = second
@@ -136,7 +137,7 @@ class MessagePassing(torch.nn.Module):
         return node_features
 
 
-def test():
+def test() -> None:
     from torch_cluster import radius_graph
     from e3nn.util.test import assert_equivariant, assert_auto_jitable
 
