@@ -1,5 +1,5 @@
-r"""Spherical Harmonics as functions of Euler angles
-"""
+r"""Spherical Harmonics as functions of Euler angles"""
+
 import math
 from typing import List, Tuple
 
@@ -22,7 +22,7 @@ class SphericalHarmonicsAlphaBeta(torch.nn.Module):
     _ls_list: List[int]
     _lmax: int
 
-    def __init__(self, l, normalization="integral"):
+    def __init__(self, l, normalization: str = "integral") -> None:
         super().__init__()
 
         if isinstance(l, o3.Irreps):
@@ -59,7 +59,7 @@ class SphericalHarmonicsAlphaBeta(torch.nn.Module):
         return out.reshape(alpha.shape + (shy.shape[1],))
 
 
-def spherical_harmonics_alpha_beta(l, alpha, beta, *, normalization="integral"):
+def spherical_harmonics_alpha_beta(l, alpha, beta, *, normalization: str = "integral"):
     r"""Spherical harmonics of :math:`\vec r = R_y(\alpha) R_x(\beta) e_y`
 
     .. math:: Y^l(\alpha, \beta) = S^l(\alpha) P^l(\cos(\beta))
@@ -127,7 +127,7 @@ def spherical_harmonics_alpha(l: int, alpha: torch.Tensor) -> torch.Tensor:
 class Legendre(fx.GraphModule):
     # pylint: disable=abstract-method
 
-    def __init__(self, ls):
+    def __init__(self, ls) -> None:
         super().__init__(self, fx.Graph())
 
         # == generate code ==
@@ -171,7 +171,7 @@ def _poly_legendre(l, m):
     return Poly(_sympy_legendre(l, m), domain="R", gens=(z, y)).as_dict()
 
 
-def _sympy_legendre(l, m):
+def _sympy_legendre(l, m) -> float:
     r"""
     en.wikipedia.org/wiki/Associated_Legendre_polynomials
     - remove two times (-1)^m
