@@ -1,6 +1,6 @@
 import itertools
 import collections
-from typing import List, Union
+from typing import List, Union, Callable
 
 import torch
 
@@ -58,6 +58,9 @@ class Irrep(tuple):
             if isinstance(l, Irrep):
                 return l
 
+            if isinstance(l, _MulIr):
+                return l.ir.l
+
             if isinstance(l, str):
                 try:
                     name = l.strip()
@@ -89,7 +92,7 @@ class Irrep(tuple):
         r"""The parity of the representation, :math:`p = \pm 1`."""
         return self[1]
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         p = {+1: "e", -1: "o"}[self.p]
         return f"{self.l}{p}"
 
@@ -110,7 +113,7 @@ class Irrep(tuple):
             if l == lmax:
                 break
 
-    def D_from_angles(self, alpha, beta, gamma, k=None):
+    def D_from_angles(self, alpha, beta, gamma, k=None) -> torch.Tensor:
         r"""Matrix :math:`p^k D^l(\alpha, \beta, \gamma)`
 
         (matrix) Representation of :math:`O(3)`. :math:`D` is the representation of :math:`SO(3)`, see `wigner_D`.
@@ -149,7 +152,7 @@ class Irrep(tuple):
         alpha, beta, gamma, k = torch.broadcast_tensors(alpha, beta, gamma, k)
         return _wigner.wigner_D(self.l, alpha, beta, gamma) * self.p ** k[..., None, None]
 
-    def D_from_quaternion(self, q, k=None):
+    def D_from_quaternion(self, q, k=None) -> torch.Tensor:
         r"""Matrix of the representation, see `Irrep.D_from_angles`
 
         Parameters
@@ -167,7 +170,7 @@ class Irrep(tuple):
         """
         return self.D_from_angles(*_rotation.quaternion_to_angles(q), k)
 
-    def D_from_matrix(self, R):
+    def D_from_matrix(self, R) -> torch.Tensor:
         r"""Matrix of the representation, see `Irrep.D_from_angles`
 
         Parameters
@@ -196,7 +199,7 @@ class Irrep(tuple):
         k = (1 - d) / 2
         return self.D_from_angles(*_rotation.matrix_to_angles(R), k)
 
-    def D_from_axis_angle(self, axis, angle):
+    def D_from_axis_angle(self, axis, angle) -> torch.Tensor:
         r"""Matrix of the representation, see `Irrep.D_from_angles`
 
         Parameters
@@ -282,7 +285,7 @@ class _MulIr(tuple):
     def dim(self) -> int:
         return self.mul * self.ir.dim
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return f"{self.mul}x{self.ir}"
 
     def __getitem__(self, item) -> Union[int, Irrep]:  # pylint: disable=useless-super-delegation
@@ -396,7 +399,7 @@ class Irreps(tuple):
         return super().__new__(cls, out)
 
     @staticmethod
-    def spherical_harmonics(lmax, p=-1):
+    def spherical_harmonics(lmax: int, p: int = -1) -> "Irreps":
         r"""representation of the spherical harmonics
 
         Parameters
@@ -439,7 +442,9 @@ class Irreps(tuple):
             i += mul_ir.dim
         return s
 
-    def randn(self, *size, normalization="component", requires_grad=False, dtype=None, device=None):
+    def randn(
+        self, *size: int, normalization: str = "component", requires_grad: bool = False, dtype=None, device=None
+    ) -> torch.Tensor:
         r"""Random tensor.
 
         Parameters
@@ -509,11 +514,11 @@ class Irreps(tuple):
     def index(self, _object):
         raise NotImplementedError
 
-    def __add__(self, irreps):
+    def __add__(self, irreps) -> "Irreps":
         irreps = Irreps(irreps)
         return Irreps(super().__add__(irreps))
 
-    def __mul__(self, other):
+    def __mul__(self, other) -> "Irreps":
         r"""
         >>> (Irreps('2x1e') * 3).simplify()
         6x1e
@@ -522,14 +527,14 @@ class Irreps(tuple):
             raise NotImplementedError("Use o3.TensorProduct for this, see the documentation")
         return Irreps(super().__mul__(other))
 
-    def __rmul__(self, other):
+    def __rmul__(self, other) -> "Irreps":
         r"""
         >>> 2 * Irreps('0e + 1e')
         1x0e+1x1e+1x0e+1x1e
         """
         return Irreps(super().__rmul__(other))
 
-    def simplify(self) -> 'Irreps':
+    def simplify(self) -> "Irreps":
         """Simplify the representations.
 
         Returns
@@ -557,7 +562,7 @@ class Irreps(tuple):
                 out.append((mul, ir))
         return Irreps(out)
 
-    def remove_zero_multiplicities(self):
+    def remove_zero_multiplicities(self) -> "Irreps":
         """Remove any irreps with multiplicities of zero.
 
         Returns
@@ -603,6 +608,95 @@ class Irreps(tuple):
         irreps = Irreps([(mul, ir) for ir, _, mul in out])
         return Ret(irreps, p, inv)
 
+    def regroup(self) -> "Irreps":
+        r"""Regroup the same irreps together.
+
+        Equivalent to :meth:`sort` followed by :meth:`simplify`.
+
+        Returns
+        -------
+            irreps: `e3nn.o3.Irreps`
+
+        Examples
+        --------
+
+        >>> Irreps("1e + 0e + 1e + 0x2e").regroup()
+        1x0e+2x1e
+        """
+        return self.sort().irreps.simplify()
+
+    def filter(
+        self,
+        keep: Union["Irreps", List[Irrep], Callable[[_MulIr], bool]] = None,
+        *,
+        drop: Union["Irreps", List[Irrep], Callable[[_MulIr], bool]] = None,
+        lmax: int = None,
+    ) -> "Irreps":
+        r"""Filter the irreps.
+
+        Args:
+            keep (`Irreps` or list of `Irrep` or function): list of irrep to keep
+            drop (`Irreps` or list of `Irrep` or function): list of irrep to drop
+            lmax (int): maximum :math:`l` value
+
+        Returns:
+            `Irreps`: filtered irreps
+
+        Examples:
+            >>> Irreps("1e + 2e + 0e").filter(keep=["0e", "1e"])
+            1x1e+1x0e
+
+            >>> Irreps("1e + 2e + 0e").filter(keep="2e + 2x1e")
+            1x1e+1x2e
+
+            >>> Irreps("1e + 2e + 0e").filter(drop="2e + 2x1e")
+            1x0e
+
+            >>> Irreps("1e + 2e + 0e").filter(lmax=1)
+            1x1e+1x0e
+        """
+        if keep is None and drop is None and lmax is None:
+            return self
+        if keep is not None and drop is not None:
+            raise ValueError("Cannot specify both keep and drop")
+        if keep is not None and lmax is not None:
+            raise ValueError("Cannot specify both keep and lmax")
+        if drop is not None and lmax is not None:
+            raise ValueError("Cannot specify both drop and lmax")
+
+        if keep is not None:
+            if isinstance(keep, str):
+                keep = Irreps(keep)
+            if isinstance(keep, Irrep):
+                keep = [keep]
+            if isinstance(keep, _MulIr):
+                keep = [keep.ir]
+            if callable(keep):
+                return Irreps([mul_ir for mul_ir in self if keep(mul_ir)])
+            keep = {Irrep(ir) for ir in keep}
+            return Irreps([(mul, ir) for mul, ir in self if ir in keep])
+
+        if drop is not None:
+            if isinstance(drop, str):
+                drop = Irreps(drop)
+            if isinstance(drop, Irrep):
+                drop = [drop]
+            if isinstance(drop, _MulIr):
+                drop = [drop.ir]
+            if callable(drop):
+                return Irreps([mul_ir for mul_ir in self if not drop(mul_ir)])
+            drop = {Irrep(ir) for ir in drop}
+            return Irreps([(mul, ir) for mul, ir in self if ir not in drop])
+
+        if lmax is not None:
+            return Irreps([(mul, ir) for mul, ir in self if ir.l <= lmax])
+
+    @property
+    def slice_by_mul(self):
+        r"""Return the slice with respect to the multiplicities.
+        """
+        return _MulIndexSliceHelper(self)
+
     @property
     def dim(self) -> int:
         return sum(mul * ir.dim for mul, ir in self)
@@ -621,7 +715,7 @@ class Irreps(tuple):
             raise ValueError("Cannot get lmax of empty Irreps")
         return max(self.ls)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return "+".join(f"{mul_ir}" for mul_ir in self)
 
     def D_from_angles(self, alpha, beta, gamma, k=None):
@@ -646,7 +740,11 @@ class Irreps(tuple):
         `torch.Tensor`
             tensor of shape :math:`(..., \mathrm{dim}, \mathrm{dim})`
         """
-        return direct_sum(*[ir.D_from_angles(alpha, beta, gamma, k) for mul, ir in self for _ in range(mul)])
+        blocks = []
+        for mul, ir in self:
+            D = ir.D_from_angles(alpha, beta, gamma, k)
+            blocks.extend([D] * mul)
+        return direct_sum(*blocks)
 
     def D_from_quaternion(self, q, k=None):
         r"""Matrix of the representation
@@ -701,3 +799,28 @@ class Irreps(tuple):
             tensor of shape :math:`(..., \mathrm{dim}, \mathrm{dim})`
         """
         return self.D_from_angles(*_rotation.axis_angle_to_angles(axis, angle))
+
+
+class _MulIndexSliceHelper:
+    irreps: Irreps
+
+    def __init__(self, irreps) -> None:
+        self.irreps = irreps
+
+    def __getitem__(self, index: slice) -> Irreps:
+        if not isinstance(index, slice):
+            raise IndexError("Irreps.slice_by_mul only supports slices.")
+
+        start, stop, stride = index.indices(self.irreps.num_irreps)
+        if stride != 1:
+            raise NotImplementedError("Irreps.slice_by_mul does not support strides.")
+
+        out = []
+        i = 0
+        for mul, ir in self.irreps:
+            if start <= i and i + mul <= stop:
+                out.append((mul, ir))
+            elif start < i + mul and i < stop:
+                out.append((min(stop, i + mul) - max(start, i), ir))
+            i += mul
+        return Irreps(out)
