@@ -1,14 +1,13 @@
-r"""Spherical Harmonics as polynomials of x, y, z
-"""
+r"""Spherical Harmonics as polynomials of x, y, z"""
+
 from typing import Union, List, Any
 
 import math
 
-import sympy
-from sympy.printing.pycode import pycode
 import torch
 
-from e3nn import o3
+from e3nn.o3._irreps import Irreps
+from e3nn import get_optimization_defaults
 from e3nn.util.jit import compile_mode
 
 
@@ -28,27 +27,27 @@ class SphericalHarmonics(torch.nn.Module):
 
     def __init__(
         self,
-        irreps_out: Union[int, List[int], str, o3.Irreps],
+        irreps_out: Union[int, List[int], str, Irreps],
         normalize: bool,
         normalization: str = "integral",
         irreps_in: Any = None,
-    ):
+    ) -> None:
         super().__init__()
         self.normalize = normalize
         self.normalization = normalization
         assert normalization in ["integral", "component", "norm"]
 
         if isinstance(irreps_out, str):
-            irreps_out = o3.Irreps(irreps_out)
-        if isinstance(irreps_out, o3.Irreps) and irreps_in is None:
+            irreps_out = Irreps(irreps_out)
+        if isinstance(irreps_out, Irreps) and irreps_in is None:
             for mul, (l, p) in irreps_out:
                 if l % 2 == 1 and p == 1:
-                    irreps_in = o3.Irreps("1e")
+                    irreps_in = Irreps("1e")
         if irreps_in is None:
-            irreps_in = o3.Irreps("1o")
+            irreps_in = Irreps("1o")
 
-        irreps_in = o3.Irreps(irreps_in)
-        if irreps_in not in (o3.Irreps("1x1o"), o3.Irreps("1x1e")):
+        irreps_in = Irreps(irreps_in)
+        if irreps_in not in (Irreps("1x1o"), Irreps("1x1e")):
             raise ValueError(
                 f"irreps_in for SphericalHarmonics must be either a vector (`1x1o`) or a pseudovector (`1x1e`), "
                 f"not `{irreps_in}`"
@@ -56,7 +55,7 @@ class SphericalHarmonics(torch.nn.Module):
         self.irreps_in = irreps_in
         input_p = irreps_in[0].ir.p  # pylint: disable=no-member
 
-        if isinstance(irreps_out, o3.Irreps):
+        if isinstance(irreps_out, Irreps):
             ls = []
             for mul, (l, p) in irreps_out:
                 if p != input_p**l:
@@ -71,7 +70,7 @@ class SphericalHarmonics(torch.nn.Module):
         else:
             ls = list(irreps_out)
 
-        irreps_out = o3.Irreps([(1, (l, input_p**l)) for l in ls]).simplify()
+        irreps_out = Irreps([(1, (l, input_p**l)) for l in ls]).simplify()
         self.irreps_out = irreps_out
         self._ls_list = ls
         self._lmax = max(ls)
@@ -84,12 +83,19 @@ class SphericalHarmonics(torch.nn.Module):
                 f"spherical_harmonics maximum l implemented is {_lmax}, send us an email to ask for more"
             )
 
+        if get_optimization_defaults()["jit_mode"] == "script":
+            self.sph_func = torch.jit.script(_spherical_harmonics)
+        elif get_optimization_defaults()["jit_mode"] == "compile":
+            self.sph_func = torch.compile(_spherical_harmonics, fullgraph=True)
+        else:
+            self.sph_func = _spherical_harmonics
+
     def forward(self, x: torch.Tensor) -> torch.Tensor:
         # - PROFILER - with torch.autograd.profiler.record_function(self._prof_str):
         if self.normalize:
             x = torch.nn.functional.normalize(x, dim=-1)  # forward 0's instead of nan for zero-radius
 
-        sh = _spherical_harmonics(self._lmax, x[..., 0], x[..., 1], x[..., 2])
+        sh = self.sph_func(self._lmax, x[..., 0], x[..., 1], x[..., 2])
 
         if not self._is_range_lmax:
             sh = torch.cat([sh[..., l * l : (l + 1) * (l + 1)] for l in self._ls_list], dim=-1)
@@ -107,7 +113,7 @@ class SphericalHarmonics(torch.nn.Module):
 
 
 def spherical_harmonics(
-    l: Union[int, List[int], str, o3.Irreps], x: torch.Tensor, normalize: bool, normalization: str = "integral"
+    l: Union[int, List[int], str, Irreps], x: torch.Tensor, normalize: bool, normalization: str = "integral"
 ):
     r"""Spherical harmonics
 
@@ -184,7 +190,6 @@ def spherical_harmonics(
     return sh(x)
 
 
-@torch.jit.script
 def _spherical_harmonics(lmax: int, x: torch.Tensor, y: torch.Tensor, z: torch.Tensor) -> torch.Tensor:
     sh_0_0 = torch.ones_like(x)
     if lmax == 0:
@@ -1941,65 +1946,3 @@ def _spherical_harmonics(lmax: int, x: torch.Tensor, y: torch.Tensor, z: torch.T
         ],
         dim=-1,
     )
-
-
-def _generate_spherical_harmonics(lmax, device=None):  # pragma: no cover
-    r"""code used to generate the code above
-
-    based on `wigner_3j`
-    """
-    torch.set_default_dtype(torch.float64)
-
-    def to_frac(x: float):
-        from fractions import Fraction
-
-        s = 1 if x >= 0 else -1
-        x = x**2
-        x = Fraction(x).limit_denominator()
-        x = s * sympy.sqrt(x)
-        x = sympy.simplify(x)
-        return x
-
-    print("sh_0_0 = torch.ones_like(x)")
-    print("if lmax == 0:")
-    print("    return torch.stack([")
-    print("        sh_0_0,")
-    print("    ], dim=-1)")
-    print()
-
-    x_var, y_var, z_var = sympy.symbols("x y z")
-    polynomials = [sympy.sqrt(3) * x_var, sympy.sqrt(3) * y_var, sympy.sqrt(3) * z_var]
-
-    def sub_z1(p, names, polynormz):
-        p = p.subs(x_var, 0).subs(y_var, 1).subs(z_var, 0)
-        for n, c in zip(names, polynormz):
-            p = p.subs(n, c)
-        return p
-
-    poly_evalz = [sub_z1(p, [], []) for p in polynomials]
-
-    for l in range(1, lmax + 1):
-        sh_variables = sympy.symbols(" ".join(f"sh_{l}_{m}" for m in range(2 * l + 1)))
-
-        for n, p in zip(sh_variables, polynomials):
-            print(f"{n} = {pycode(p)}")
-
-        print(f"if lmax == {l}:")
-        u = ",\n        ".join(", ".join(f"sh_{j}_{m}" for m in range(2 * j + 1)) for j in range(l + 1))
-        print(f"    return torch.stack([\n        {u}\n    ], dim=-1)")
-        print()
-
-        if l == lmax:
-            break
-
-        polynomials = [
-            sum(to_frac(c.item()) * v * sh for cj, v in zip(cij, [x_var, y_var, z_var]) for c, sh in zip(cj, sh_variables))
-            for cij in o3.wigner_3j(l + 1, 1, l, device=device)
-        ]
-
-        poly_evalz = [sub_z1(p, sh_variables, poly_evalz) for p in polynomials]
-        norm = sympy.sqrt(sum(p**2 for p in poly_evalz))
-        polynomials = [sympy.sqrt(2 * l + 3) * p / norm for p in polynomials]
-        poly_evalz = [sympy.sqrt(2 * l + 3) * p / norm for p in poly_evalz]
-
-        polynomials = [sympy.simplify(p, full=True) for p in polynomials]
